@@ -1,4 +1,4 @@
-use crate::util::hash::{decode, encode, Hash, Hashable};
+use crate::util::hash::{b58_decode, b58_encode, Hash, Hashable};
 use anyhow::{anyhow, Result};
 use serde::de::Visitor;
 use serde::{de, ser, Deserialize, Deserializer, Serialize, Serializer};
@@ -15,6 +15,7 @@ pub enum Address {
     Hash(Hash),
     Uuid(Uuid),
     Attribute(String),
+    Url(String),
 }
 
 // multihash KangarooTwelve
@@ -29,6 +30,7 @@ impl Address {
             Self::Hash(hash) => (KANGAROO_TWELVE, hash.0.clone()),
             Self::Uuid(uuid) => (IDENTITY, [vec![b'U'], uuid.as_bytes().to_vec()].concat()),
             Self::Attribute(attribute) => (IDENTITY, [&[b'A'], attribute.as_bytes()].concat()),
+            Self::Url(url) => (IDENTITY, [&[b'X'], url.as_bytes()].concat()),
         };
 
         let mut result = Cursor::new(vec![0u8; 0]);
@@ -59,6 +61,7 @@ impl Address {
                             digest_content.as_slice(),
                         )?)),
                         b'A' => Ok(Self::Attribute(String::from_utf8(digest_content)?)),
+                        b'X' => Ok(Self::Url(String::from_utf8(digest_content)?)),
                         _ => Err(anyhow!("Unknown identity marker.")),
                     }
                 }
@@ -73,7 +76,7 @@ impl Serialize for Address {
     where
         S: Serializer,
     {
-        serializer.serialize_str(encode(self.encode().map_err(ser::Error::custom)?).as_str())
+        serializer.serialize_str(b58_encode(self.encode().map_err(ser::Error::custom)?).as_str())
     }
 }
 
@@ -90,7 +93,7 @@ impl<'de> Visitor<'de> for AddressVisitor {
     where
         E: de::Error,
     {
-        let bytes = decode(str).map_err(de::Error::custom)?;
+        let bytes = b58_decode(str).map_err(de::Error::custom)?;
         Address::decode(bytes.as_ref()).map_err(de::Error::custom)
     }
 }
@@ -108,13 +111,17 @@ impl FromStr for Address {
     type Err = anyhow::Error;
 
     fn from_str(s: &str) -> Result<Self, Self::Err> {
-        Address::decode(decode(s)?.as_ref())
+        Address::decode(b58_decode(s)?.as_ref())
     }
 }
 
 impl std::fmt::Display for Address {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
-        write!(f, "{}", encode(self.encode().map_err(|_| std::fmt::Error)?))
+        write!(
+            f,
+            "{}",
+            b58_encode(self.encode().map_err(|_| std::fmt::Error)?)
+        )
     }
 }
 
@@ -132,47 +139,45 @@ pub trait Addressable: Hashable {
 
 #[cfg(test)]
 mod tests {
+    use anyhow::Result;
     use uuid::Uuid;
 
     use crate::addressing::Address;
     use crate::util::hash::Hash;
 
     #[test]
-    fn test_hash_codec() {
+    fn test_hash_codec() -> Result<()> {
         let addr = Address::Hash(Hash(vec![1, 2, 3, 4, 5]));
-
-        let encoded = addr.encode();
-        assert!(encoded.is_ok());
-
-        let decoded = Address::decode(&encoded.unwrap());
-        assert!(decoded.is_ok());
-
-        assert_eq!(format!("{}", addr), format!("{}", decoded.unwrap()));
+        let encoded = addr.encode()?;
+        let decoded = Address::decode(&encoded)?;
+        assert_eq!(addr, decoded);
+        Ok(())
     }
 
     #[test]
-    fn test_uuid_codec() {
+    fn test_uuid_codec() -> Result<()> {
         let addr = Address::Uuid(Uuid::new_v4());
-
-        let encoded = addr.encode();
-        assert!(encoded.is_ok());
-
-        let decoded = Address::decode(&encoded.unwrap());
-        assert!(decoded.is_ok());
-
-        assert_eq!(format!("{}", addr), format!("{}", decoded.unwrap()));
+        let encoded = addr.encode()?;
+        let decoded = Address::decode(&encoded)?;
+        assert_eq!(addr, decoded);
+        Ok(())
     }
 
     #[test]
-    fn test_attribute_codec() {
+    fn test_attribute_codec() -> Result<()> {
         let addr = Address::Attribute(String::from("ATTRIBUTE"));
+        let encoded = addr.encode()?;
+        let decoded = Address::decode(&encoded)?;
+        assert_eq!(addr, decoded);
+        Ok(())
+    }
 
-        let encoded = addr.encode();
-        assert!(encoded.is_ok());
-
-        let decoded = Address::decode(&encoded.unwrap());
-        assert!(decoded.is_ok());
-
-        assert_eq!(format!("{}", addr), format!("{}", decoded.unwrap()));
+    #[test]
+    fn test_url_codec() -> Result<()> {
+        let addr = Address::Url(String::from("https://upendproject.net"));
+        let encoded = addr.encode()?;
+        let decoded = Address::decode(&encoded)?;
+        assert_eq!(addr, decoded);
+        Ok(())
     }
 }
