@@ -3,7 +3,7 @@ use std::path::PathBuf;
 use chrono::NaiveDateTime;
 use serde::Serialize;
 
-use super::schema::{data, files};
+use super::schema::{data, files, meta};
 use crate::util::hash::Hash;
 
 #[derive(Queryable, Serialize, Clone, Debug)]
@@ -49,3 +49,11 @@ pub struct Entry {
     pub value_num: Option<f64>,
     pub immutable: bool,
 }
+
+#[derive(Queryable, Insertable, Serialize, Clone, Debug)]
+#[table_name = "meta"]
+pub struct MetaValue {
+    pub id: i32,
+    pub key: String,
+    pub value: String,
+}
