@@ -23,14 +23,10 @@ table! {
 
 table! {
     meta (id) {
-        id -> Nullable<Integer>,
+        id -> Integer,
         key -> Text,
         value -> Text,
     }
 }
 
-allow_tables_to_appear_in_same_query!(
-    data,
-    files,
-    meta,
-);
+allow_tables_to_appear_in_same_query!(data, files, meta,);
