@@ -12,10 +12,12 @@ use std::{
 use self::image::ImagePath;
 use self::text::TextPath;
 use self::video::VideoPath;
+use self::audio::AudioPath;
 
 pub mod image;
 pub mod text;
 pub mod video;
+pub mod audio;
 
 pub trait Previewable {
     fn get_thumbnail(&self) -> Result<Option<Vec<u8>>>;
@@ -60,9 +62,12 @@ impl PreviewStore {
             if let Some(file) = files.get(0) {
                 let preview = match tree_magic_mini::from_filepath(&file.path) {
                     Some(tm) if tm.starts_with("text") => Ok(TextPath(&file.path).get_thumbnail()?),
-                    Some(tm) if tm.starts_with("video") => {
+                    Some(tm) if tm.starts_with("video") || tm == "application/x-matroska" => {
                         Ok(VideoPath(&file.path).get_thumbnail()?)
                     }
+                    Some(tm) if tm.starts_with("audio") => {
+                        Ok(AudioPath(&file.path).get_thumbnail()?)
+                    }
                     Some(tm) if tm.starts_with("image") => {
                         Ok(ImagePath(&file.path).get_thumbnail()?)
                     }
