@@ -72,14 +72,16 @@ pub async fn get_raw(
             let file_path = state.upend.vault_path.join(&file.path);
 
             if query.native.is_none() {
-                Ok(Either::A(if query.inline.is_some() {
+                Ok(Either::A(
                     NamedFile::open(file_path)?.set_content_disposition(ContentDisposition {
-                        disposition: DispositionType::Inline,
+                        disposition: if query.inline.is_some() {
+                            DispositionType::Inline
+                        } else {
+                            DispositionType::Attachment
+                        },
                         parameters: vec![],
-                    })
-                } else {
-                    NamedFile::open(file_path)?
-                }))
+                    }),
+                ))
             } else if state.desktop_enabled {
                 #[cfg(feature = "desktop")]
                 {
